//
// Copyright (c) 2009, Markus Rickert
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
// * Neither the name of the Technische Universitaet Muenchen nor the names of
//   its contributors may be used to endorse or promote products derived from
//   this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

#ifndef _RL_MDL_ELEMENT_H_
#define _RL_MDL_ELEMENT_H_

#include <string>
#include <rl/math/Real.h>

namespace rl
{
	namespace mdl
	{
		class Compound;
		
		class Element
		{
		public:
			Element();
			
			virtual ~Element();
			
			virtual void forwardAcceleration() = 0;
			
			virtual void forwardDynamics1() = 0;
			
			virtual void forwardDynamics2() = 0;
			
			virtual void forwardDynamics3() = 0;
			
			virtual void forwardPosition() = 0;
			
			virtual void forwardVelocity() = 0;
			
			const ::std::string& getName() const;
			
			virtual void inverseDynamics1() = 0;
			
			virtual void inverseDynamics2() = 0;
			
			virtual void inverseForce() = 0;
			
			virtual void inverseVelocity1(const ::rl::math::Real& lambda) = 0;
			
			virtual void inverseVelocity2() = 0;
			
			virtual void inverseVelocity3() = 0;
			
			void setName(const ::std::string& name);
			
			Compound* compound;
			
		protected:
			
		private:
			::std::string name;
		};
	}
}

#endif // _RL_MDL_ELEMENT_H_
